[![Build Status](https://travis-ci.org/Automattic/juice.png?branch=master)](https://travis-ci.org/Automattic/juice)
[![Dependency Status](https://david-dm.org/Automattic/juice.png)](https://david-dm.org/Automattic/juice)
# Juice ![](http://i.imgur.com/jN8Ht.gif)

Given HTML, juice will inline your CSS properties into the `style` attribute.

## How to use

Juice has a number of functions based on whether you want to process a file, HTML string, or a cheerio document, and whether you want juice to automatically get remote stylesheets, scripts and image dataURIs to inline.

To inline HTML without getting remote resources, using default options:

```js
var juice = require('juice');
var result = juice("<style>div{color:red;}</style><div/>");
```

result will be:
```html
<div style="color: red;"></div>
```

## What is this useful for ?

- HTML emails. For a comprehensive list of supported selectors see [here](http://www.campaignmonitor.com/css/)
- Embedding HTML in 3rd-party websites.

## Projects using juice

* [node-email-templates][1] - Node.js module for rendering beautiful emails with [ejs][2] templates and email-friendly inline CSS using [juice][3].
* [swig-email-templates][4] - Uses [swig][5], which gives you [template inheritance][6], and can generate a [dummy context][7] from a template.

[1]: https://github.com/niftylettuce/node-email-templates
[2]: https://github.com/visionmedia/ejs
[3]: https://github.com/Automattic/juice
[4]: https://github.com/superjoe30/swig-email-templates
[5]: https://github.com/paularmstrong/swig
[6]: https://docs.djangoproject.com/en/dev/topics/templates/#template-inheritance
[7]: https://github.com/superjoe30/swig-dummy-context

## Documentation

Juice is exposed as a standard module, and from CLI with a smaller set of options.

### Options

All juice methods take an options object that can contain any of these properties, though not every method uses all of these:

 * `extraCss` - extra css to apply to the file. Defaults to `""`.
 * `applyStyleTags` - whether to inline styles in `<style></style>` Defaults to `true`.
 * `removeStyleTags` - whether to remove the original `<style></style>` tags after (possibly) inlining the css from them. Defaults to `true`.
 * `preserveMediaQueries` - preserves all media queries (and contained styles) within `<style></style>` tags as a refinement when `removeStyleTags` is `true`. Other styles are removed. Defaults to `false`.
  * `preserveFontFaces` - preserves all `@font-face` within `<style></style>` tags as a refinement when `removeStyleTags` is `true`. Other styles are removed. Defaults to `false`.
 * `applyWidthAttributes` - whether to use any CSS pixel widths to create `width` attributes on elements set in `juice.widthElements`. Defaults to `false`.
 * `applyAttributesTableElements` - whether to create attributes for styles in `juice.styleToAttribute` on elements set in `juice.tableElements`. Defaults to `false`.
 * `webResources` - An options object that will be passed to [web-resource-inliner](https://www.npmjs.com/package/web-resource-inliner) for juice functions that will get remote resources (`juiceResources` and `juiceFile`). Defaults to `{}`.
 * `inlinePseudoElements` - Whether to insert pseudo elements (`::before` and `::after`) as `<span>` into the DOM. *Note*: Inserting pseudo elements will modify the DOM and may conflict with CSS selectors elsewhere on the page (e.g., `:last-child`).
 * `xmlMode` - whether to output XML/XHTML with all tags closed. Note that the input *must* also be valid XML/XHTML or you will get undesirable results. Defaults to `false`.
 * `preserveImportant` - preserves `!important` in values. Defaults to `false`.

### Methods

#### juice(html [, options])

Returns string containing inlined HTML. Does not fetch remote resources.

 * `html` - html string, accepts complete documents as well as fragments
 * `options` - optional, see Options above

#### juice.juiceResources(html, options, callback)

Callback returns string containing inlined HTML. Fetches remote resources.

 * `html` - html string
 * `options` - see Options above
 * `callback(err, html)`
   - `err` - `Error` object or `null`
   - `html` - inlined HTML

#### juice.juiceFile(filePath, options, callback)

Callback returns string containing inlined HTML. Fetches remote resources.

 * `filePath` - path to the html file to be juiced
 * `options` - see Options above
 * `callback(err, html)`
   - `err` - `Error` object or `null`
   - `html` - inlined HTML

#### juice.juiceDocument($ [, options])

Returns string containing inlined HTML. Does not fetch remote resources.

 * `$` - a cheerio instance, be sure to use the same cheerio version that juice uses
 * `options` - optional, see Options above`

#### juice.inlineContent(html, css [, options])

This takes html and css and returns new html with the provided css inlined.
It does not look at `<style>` or `<link rel="stylesheet">` elements at all.

 * `html` - html string
 * `css` - css string
 * `options` - optional, see Options above

#### juice.inlineDocument($, css [, options])

Given a cheerio instance and css, this modifies the cheerio instance so that the provided css is inlined. It does not look at `<style>` or `<link rel="stylesheet">` elements at all.

 * `$` - a cheerio instance, be sure to use the same cheerio version that juice uses
 * `css` - css string
 * `options` - optional, see Options above

### Global settings

#### juice.ignoredPseudos

Array of ignored pseudo-selectors such as 'hover' and 'active'.

#### juice.widthElements

Array of HTML elements that can receive `width` attributes.

#### juice.styleToAttribute

Object of style property names (key) to their respective attribute names (value).

#### juice.tableElements

Array of table HTML elements that can receive attributes defined in `juice.styleToAttribute`.

#### juice.nonVisualElements

Array of elements that will not have styles inlined because they are not intended to render.


### CLI Options

To use Juice from CLI, run `juice [options] input.html output.html`

Current CLI options:

- `--css [filepath]` will load and inject CSS into `extraCss`.


## Credits

(The MIT License)

Copyright (c) 2015 Guillermo Rauch &lt;guillermo@learnboost.com&gt;

Permission is hereby granted, free of charge, to any person obtaining
a copy of this software and associated documentation files (the
'Software'), to deal in the Software without restriction, including
without limitation the rights to use, copy, modify, merge, publish,
distribute, sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to
the following conditions:

The above copyright notice and this permission notice shall be
included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

### 3rd-party

- Uses [cheerio](https://github.com/cheeriojs/cheerio) for the underlying DOM
representation.
- Uses [cssom](https://github.com/NV/CSSOM) to parse out CSS selectors and
[Slick](http://github.com/subtleGradient/slick) to tokenize them.
- Icon by [UnheardSounds](http://unheardsounds.deviantart.com/gallery/26536908#/d2ngozi)
